{% extends "base.html" %}

{% block title %}
	&raquo; Insufficient Permissions
{% endblock %}

{% block nav %}
	Insufficient Permissions
{% endblock %}

{% block content %}
	<h2>Insufficient Permissions</h2>

	{% set msg = properties.forbidden_message %}
	<p>{% if msg %}{{msg}}{% else %}Insufficient Permissions{% endif %}</p>

{% endblock %}
