{% extends "base.html" %}

{% block title %}
	Asset Comparison
{% endblock %}

{% block nav %}
	Asset Comparison
{% endblock %}

{% block content %}

	<form enctype="multipart/form-data" method="post"
			action="/reports/asset_comparison">
		<fieldset>
			<legend>Comparison Against Asset Data</legend>	
			Asset File<input type="file" name="asset_file" size="45"
		 		value="{{request.values.asset_file}}">
			<input type="submit" value="Compare">
		</fieldset>
	</form>

	<p>
		This tool takes a spreadsheet of asset data and compares it against Chellow data
	 	and reports any inconsistencies. The spreadsheet can be downloaded from
	 	{{description}}.
	</p>

{% endblock %}
