{% extends "base.html" %}

{% block title %}
	&raquo; Energisation Statuses
{% endblock %}

{% block nav %}
	Energisation Statuses
{% endblock %}

{% block content %}

	<table>
		<thead>
			<tr>
				<th>Code</th>
				<th>Description</th>
			</tr>
		</thead>
		<tbody>
			{% for energisation_status in energisation_statuses %}
				<tr>
					<td>
						<a
								href="/e/energisation_statuses/{{energisation_status.id}}"
								>{{energisation_status.code}}</a>
					</td>
					<td>{{energisation_status.description}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>

{% endblock %}
